/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "RMatrix.hh"
#include <Math/SVD.hh>
#include <Math/Quaternion.hh>

using namespace BIAS;
using namespace std;




RMatrix::~RMatrix()
{}

RMatrix::RMatrix(const TNT::Matrix<ROTATION_MATRIX_TYPE>& mat)
  : RMatrixBase(mat)
{
  if ((mat.num_rows() != 3) || (mat.num_cols() != 3)) {
    BIASERR("wrong matrix size "<<mat.num_rows()<<"x"
           << mat.num_cols());
  }
}

int RMatrix::EnforceRConstraints()
{
  // enforce real rotation matrix using SVD
  // by setting all singular values to unity 
  SVD<double> RotSVD(Matrix<ROTATION_MATRIX_TYPE>(*this));
  *this = RotSVD.GetU() * RotSVD.GetVT();

  // enforce right-handed system (det = +1)
  if (this->GetDeterminant()<0.0) *this *= -1.0;
  return 0;
}



// -------------------- numerical L-M error computation ---------------------
vector<RMatrix> const *pGLOBALvecR = NULL;
double GLOBALbestErrorSoFar = DBL_MAX;
Quaternion<double> GLOBALbestQuaternionSoFar(0,0,0,1);

void ErrorFunction(int * /*numerrors*/, int *numvars, double *x,
                   double *fvec, int * /*iflag*/) 
{
  BIASASSERT(numvars!=NULL);
  BIASASSERT(*numvars==4);

  // create rotation matrix vector from parameterization
  Quaternion<double> q(x[0],x[1],x[2],x[3]);
  q.Normalize();
  RMatrixBase R;
  R.SetFromQuaternion(q);
 

  // compute residuum
  double error = 0, err;
  for (unsigned int j=0; j<pGLOBALvecR->size(); j++) {
    for (unsigned int row=0; row<3; row++) {
      for (unsigned int col=0; col<3; col++) {
        err = fvec[9*j+3*col+row]   
          = R[row][col] - (*pGLOBALvecR)[j][row][col];
        error += err*err;
      }
    }
  }

  // in case lm runs out of iterations, remember best in a global variable
  if (error<GLOBALbestErrorSoFar) {
    GLOBALbestErrorSoFar = error;
    GLOBALbestQuaternionSoFar = q;
  }
  cout<<error<<" ";
}
// ---------------------------------------------------------------------------






