/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __ColourRGB_hh__
#define __ColourRGB_hh__
#include <Debug/Error.hh>
namespace BIAS {

  /** @class ColourRGB
      @brief interface class used to ease handover in function calls 
      @author woelk 09/2007 */
  template <class StorageType>
  class ColourRGB
  {
  public:
    ColourRGB() {};
    
    ColourRGB(const ColourRGB& col)
    { *this = col; }

    explicit ColourRGB(const StorageType& red, const StorageType& green, 
                       const StorageType& blue) 
    { Data_[0]=red; Data_[1]=green; Data_[2]=blue; }
    
    explicit ColourRGB(const StorageType col[3])
    { Data_[0]=col[0]; Data_[1]=col[1]; Data_[2]=col[2]; }

    ~ColourRGB() {}

    StorageType& operator[](const int i) 
    { BIASASSERT(i>=0&&i<3); return Data_[i]; }
    
    const StorageType& operator[](const int i) const
    { BIASASSERT(i>=0&&i<3); return Data_[i]; }

    ColourRGB& operator=(const ColourRGB& col)
    { Data_[0]=col.Data_[0]; Data_[1]=col.Data_[1]; 
      Data_[2]=col.Data_[2]; return *this; }
 
//     StorageType *GetData()
//     { BIASERR("deprectade, do not use"); return Data_; }

    const StorageType *GetData() const
    {  return Data_; }
  protected:
    StorageType Data_[3];
  };

#define COLOR_RED    BIAS::ColourRGB<unsigned char>(255,0,0)
#define COLOR_GREEN  BIAS::ColourRGB<unsigned char>(0,255,0)
#define COLOR_BLUE   BIAS::ColourRGB<unsigned char>(0,0,255)
#define COLOR_WHITE  BIAS::ColourRGB<unsigned char>(255,255,255)
#define COLOR_GREY50 BIAS::ColourRGB<unsigned char>(128, 128, 128)
#define COLOR_BLACK  BIAS::ColourRGB<unsigned char>(0,0,0)

} // namespace

#endif // __ColourRGB_hh__
