// testCamPoseCalib.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "Geometry/CamPoseCalib.hh"
#include "Math/Random.hh"

using namespace BIAS;
using namespace std;

int nrPoints=100;
int percentOutlier=2;
double boxSize=10, boxDepth=1, boxDistance=30; // boxSize in xy-direction
double sigmaNoise=0.10, outlier=5; // in pixels !

int _tmain(int argc, _TCHAR* argv[])
{
   cout <<"Estimating P matrix for "<<nrPoints<<" correspondences"<<endl
	   <<"with "<<sigmaNoise<< " pixel noise on the groundtruth and "<<endl<<
	   percentOutlier<<" percent outlier "<<endl<<endl;
  BIAS::Random ra;
  // create random 3D points
  std::vector<Vector3<double> > mps;
  Vector3<double> mp(3);
  for (int i=0;i<nrPoints;i++) {
    mp[0]=ra.GetUniformDistributed(-boxSize/2.0,boxSize/2.0);
    mp[1]=ra.GetUniformDistributed(-boxSize/2.0,boxSize/2.0);
    mp[2]=ra.GetUniformDistributed(boxDistance-boxDepth/2.0,
                                   boxDistance+boxDepth/2.0);    
    mps.push_back(mp);
  }
 
  double fx=912/2.0,fy=912/2.0,cx=383/2.0,cy=287/2.0;
  cx=343/2.0,cy=247/2.0;
  Vector3<double> C;
  
  CamPoseCalib estimator;

  // in order to activate debug level, define BIAS_DEBUG in bias_config.h
  // estimator.SetDebugLevel( estimator.GetDebugLevel() | 
  //	                       D_BIAS_CAMPOSE_ESTIMATE_ITER );
  //                            D_BIAS_CAMPOSE_ESTIMATE_STEPS);
  // estimator.SetDebugLevel( estimator.GetDebugLevel() | 
  //                           D_BIAS_CAMPOSE_ESTIMATE_EQSYS);
  // estimator.SetDebugLevel( estimator.GetDebugLevel() | 
  //                     D_BIAS_CAMPOSE_APPLY_SOL);
  // estimator.SetDebugLevel( estimator.GetDebugLevel() | 
  //                         D_BIAS_CAMPOSE_ESTIMATE_ITER);
  // estimator.AddDebugLevel( D_BIAS_CAMPOSE_LM );
  // estimator.SetDebugLevel( estimator.GetDebugLevel() | 
  //                       D_BIAS_CAMPOSE_EQSYS);
  // estimator.AddDebugLevel(D_BIAS_CAMPOSE_SECD);
  /////////////////////////////////////
  cout<<endl<<endl<<
    "---- estimating external parameters --------"<<endl;

  /// set the initial projection matrix
  KMatrix K;
  K.SetIdentity();
  K[0][0]= fx;               K[0][2]= cx;
  K[1][0]=   0; K[1][1]= fy; K[1][2]= cy;  // virtual image size of 320x240
  RMatrix R,resR;
  R.SetXYZ(00/180.0*M_PI,0,0);
  C.Set(0,0,0);
  PMatrix P1;
  P1.Compose(K,R,C);
  estimator.SetInitialCamera(P1);

  // if only a K matrix is set, an initial pose is automatically guessed
  //  estimator.SetInitialCamera(K);

  // now change the projection matrix a little and project the model points
  // this matrix is the ground truth
  K[0][0]= fx +5;          K[0][2]= cx+5;
  K[1][1]= fy -5;          K[1][2]= cy-0.5;
  R.SetXYZ(0, -5.0/180.0*M_PI, 0/180.0*M_PI); 
  C.Set(10,0,0);
  PMatrix P;
  P.Compose(K,R,C);
  
  std::vector<Vector2<double> > tps;
  Vector2<double> y;
  // project the model points
  for (unsigned int i=0;i<mps.size();i++) {
    HomgPoint2D(P*HomgPoint3D(mps[i])).GetEuclidian(y);
    tps.push_back(y);
  }

  // add some "noise"
  if (sigmaNoise>1E-15)
    for (unsigned int i=0;i<tps.size();i++) {
      tps[i][0]+=ra.GetNormalDistributed(0,sigmaNoise);
      tps[i][1]+=ra.GetNormalDistributed(0,sigmaNoise);
    }
 
  // and a few "outlier"
  if  (percentOutlier>0.000001) 
    for (unsigned int i=0;i<tps.size()/(percentOutlier*100);) {
      double offset=ra.GetNormalDistributed(0,outlier);
      if (fabs(offset)>(3*sigmaNoise)) {
        tps[i][0]+=offset;
        i++;
      }
      offset=ra.GetNormalDistributed(0,outlier);
      if (fabs(offset)>(3*sigmaNoise)) {
        tps[i][1]+=offset;
        i++;
      }
    }

  for (unsigned int i=0;i<mps.size();i++) {
    estimator.AddCorr(mps[i],tps[i]);
  }

  estimator.Estimate(50);

  ///////// output the result
  resR=estimator.GetRotation();
  BIAS::Vector3<double> axis;
  double angle;
  resR.GetRotationAxisAngle(axis,angle);
  cout<<"Rotation axis:"<<axis<<" angle:"<<angle/M_PI*180.0<<endl;
  cout<<"estimated trans:"<<estimator.GetTranslation()<<endl;
  PMatrix Pest;
  estimator.GetP(Pest);
  Vector2<double> yEst;
  HomgPoint2D(Pest*HomgPoint3D(mps[4])).GetEuclidian(yEst);
  cout<<"avg error:"<<estimator.GetAvgError()<<endl<<
    "tps[4]:"<<tps[4]<<endl<<"estimatedP*mps[4]:"<<yEst<<endl;
  cout<<"Real K:"<<K<<endl
      <<"Estimated K:"<<Pest.GetK()<<endl;
  


   cout<<endl<<endl<<
      "----- Continuing robust estimation with principal point and focal length ---"
        <<endl;    
    // now all 10 parameters are estimated, starting point is the 
    // last estimated P
    estimator.SetEstimatePrincipal(1);
	estimator.SetEstimateFocal(2); // estimate sx and sy seperately
	

	// enables robust estimation, assuming correspondences with 
	// more than 0.5 pixel reprojection error are outliers
    estimator.SetWeighting(.5,1);  
	estimator.SetChangeEpsilon(0,0); // forces the estimator to use all 50 iterations
    estimator.Estimate(50);

    resR=estimator.GetRotation();
    resR.GetRotationAxisAngle(axis,angle);
    cout<<"Rotation axis:"<<axis<<" angle:"<<angle/M_PI*180.0<<endl;
    cout<<"estimated trans:"<<estimator.GetTranslation()<<endl;
    estimator.GetP(Pest);
    HomgPoint2D(Pest*HomgPoint3D(mps[4])).GetEuclidian(yEst);
    cout<<"avg error:"<<estimator.GetAvgError()<<endl<<
      "tps[4]:"<<tps[4]<<endl<<" estimatedP*mps[4]:"<<yEst<<endl;
    cout<<"Real K:"<<K<<endl
        <<"Estimated K:"<<Pest.GetK()<<endl;
   
	cout<<endl<<"Press return to close window"<<endl;
	cin.get();
	return 0;
}

